/* config.h.  Generated by configure.  */
/* config.h.in.  Generated from configure.ac by autoheader.  */

/* Define to 1 if you have the <dlfcn.h> header file. */
#undef HAVE_DLFCN_H

/* Define to 1 if you have the <fcntl.h> header file. */
#define HAVE_FCNTL_H 1

/* Define to 1 if you have the 'getopt_long' function */
#undef HAVE_GETOPT_LONG 

/* Define to 1 if you have the <inttypes.h> header file. */
#undef HAVE_INTTYPES_H

/* Define to 1 if you have the `memmove' function. */
#undef HAVE_MEMMOVE

/* Define to 1 if you have the <memory.h> header file. */
#define HAVE_MEMORY_H 1

/* Define to 1 if you have the `random' function. */
#undef HAVE_RANDOM

/* Define to 1 if your system has a GNU libc compatible `realloc' function,
   and to 0 otherwise. */
#undef HAVE_REALLOC

/* Define to 1 if the system has the type `ssize_t'. */
#define HAVE_SSIZE_T 1

/* Define to 1 if you have the <stdint.h> header file. */
#undef HAVE_STDINT_H

/* Define to 1 if you have the <stdlib.h> header file. */
#define HAVE_STDLIB_H 1

/* Define to 1 if you have the <strings.h> header file. */
#define HAVE_STRINGS_H 1

/* Define to 1 if you have the <string.h> header file. */
#define HAVE_STRING_H 1

/* Define to 1 if you have the <sys/stat.h> header file. */
#define HAVE_SYS_STAT_H 1

/* Define to 1 if you have the <sys/types.h> header file. */
#define HAVE_SYS_TYPES_H 1

/* Define to 1 if you have the <unistd.h> header file. */
#undef HAVE_UNISTD_H

/* Define to build experimental code */
/* #undef OGGZ_CONFIG_EXPERIMENTAL */

/* Do not build reading support */
#define OGGZ_CONFIG_READ 1

/* Do not build writing support */
#define OGGZ_CONFIG_WRITE 1

/* Set to maximum allowed value of sf_count_t type. */
#define OGGZ_OFF_MAX 0x7FFFFFFFFFFFFFFFLL

/* Name of package */
#define PACKAGE "liboggz"

/* Define to the address where bug reports for this package should be sent. */
#define PACKAGE_BUGREPORT ""

/* Define to the full name of this package. */
#define PACKAGE_NAME ""

/* Define to the full name and version of this package. */
#define PACKAGE_STRING ""

/* Define to the one symbol short name of this package. */
#define PACKAGE_TARNAME ""

/* Define to the version of this package. */
#define PACKAGE_VERSION ""

/* The size of a `loff_t', as computed by sizeof. */
/* #undef SIZEOF_LOFF_T */

/* The size of a `off64_t', as computed by sizeof. */
/* #undef SIZEOF_OFF64_T */

/* The size of a `off_t', as computed by sizeof. */
/* #define SIZEOF_OFF_T 8 */

/* Set to sizeof (long) if unknown. */
/* #define SIZEOF_OGGZ_OFF_T 8 */

/* The size of a `ssize_t', as computed by sizeof. */
/* #define SIZEOF_SSIZE_T 4 */

/* Define to 1 if you have the ANSI C header files. */
#define STDC_HEADERS 1

/* Set to long if unknown. */
#define TYPEOF_OGGZ_OFF_T off_t

/* Version number of package */
#define VERSION "0.9.8"

/* Define to 1 if your processor stores words with the most significant byte
   first (like Motorola and SPARC, unlike Intel and VAX). */
/* #undef WORDS_BIGENDIAN */

/* Number of bits in a file offset, on hosts where this is settable. */
/* #undef _FILE_OFFSET_BITS */

/* Define to make fseeko etc. visible, on some hosts. */
/* #undef _LARGEFILE_SOURCE */

/* Define for large files, on AIX-style hosts. */
/* #undef _LARGE_FILES */

/* Define to empty if `const' does not conform to ANSI C. */
/* #undef const */

/* Define to empty if `const' does not conform to ANSI C. */
#undef const

/* Define to `long' if <sys/types.h> does not define. */
/* #undef off_t */

/* Define to rpl_realloc if the replacement function should be used. */
#undef realloc

/* Define to `unsigned' if <sys/types.h> does not define. */
#undef size_t

/* Define for MSVC as <stdint.h> is unavailable there */
typedef unsigned char uint8_t;

#define inline __inline // MSVC
